@@ -7,19 +7,21 @@ package Net::BitTorrent;
     use List::Util qw[max];
     use Time::HiRes;
     use Socket qw[/inet_/ SOCK_STREAM SOCK_DGRAM SOL_SOCKET PF_INET SOMAXCONN
-        /pack_sockaddr_in/];
+        /pack_sockaddr_in/ SO_REUSEADDR];
     use Carp qw[carp];
+    use Digest::SHA qw[sha1_hex];
     use POSIX qw[];
     sub _EWOULDBLOCK { $^O eq q[MSWin32] ? 10035 : POSIX::EWOULDBLOCK() }
     sub _EINPROGRESS { $^O eq q[MSWin32] ? 10036 : POSIX::EINPROGRESS() }
     use lib q[../../lib];
+    use Net::BitTorrent::Util qw[:bencode :compact];
     use Net::BitTorrent::Torrent;
     use Net::BitTorrent::Peer;
     use Net::BitTorrent::DHT;
     use Net::BitTorrent::Version;
     use version qw[qv];
-    our $SVN = q[$Id: BitTorrent.pm 42 2008-12-05 04:54:43Z sanko@cpan.org $];
-    our $UNSTABLE_RELEASE = 0; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new((qw$Rev: 42 $)[1])->numify / 1000), $UNSTABLE_RELEASE);
+    our $SVN = q[$Id: BitTorrent.pm 45 2008-12-26 22:17:16Z sanko@cpan.org $];
+    our $UNSTABLE_RELEASE = 4; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new((qw$Rev: 45 $)[1])->numify / 1000), $UNSTABLE_RELEASE);
     my (@CONTENTS)
         = \my (%_tcp,                  %_udp,
                %_schedule,             %_tid,
@@ -43,6 +45,22 @@ package Net::BitTorrent;
                     . q[parameters to be passed as a hashref];
                 return;
             }
+
+            # Defaults
+            $_peers_per_torrent{refaddr $self}    = 100;
+            $_connections_per_host{refaddr $self} = 2;
+            $_half_open{refaddr $self}            = 8;
+            $_max_dl_rate{refaddr $self}          = 0;
+            $_k_dl{refaddr $self}                 = 0;
+            $_max_ul_rate{refaddr $self}          = 0;
+            $_k_ul{refaddr $self}                 = 0;
+            $_torrents{refaddr $self}             = {};
+            $_tid{refaddr $self}                  = qq[\0] x 5;
+            $_use_dht{refaddr $self}              = 1;
+            $_dht{refaddr $self}
+                = Net::BitTorrent::DHT->new({Client => $self});
+            $_peerid{refaddr $self} = Net::BitTorrent::Version::gen_peerid();
+            $_connections{refaddr $self} = {};
             $host = $args->{q[LocalHost]}
                 if defined $args->{q[LocalHost]};
             @ports
@@ -53,22 +71,18 @@ package Net::BitTorrent;
                 )
                 : @ports;
         }
-        $_peers_per_torrent{refaddr $self}    = 100;
-        $_connections_per_host{refaddr $self} = 2;
-        $_half_open{refaddr $self}            = 8;
-        $_max_dl_rate{refaddr $self}          = 0;
-        $_k_dl{refaddr $self}                 = 0;
-        $_max_ul_rate{refaddr $self}          = 0;
-        $_k_ul{refaddr $self}                 = 0;
-        $_torrents{refaddr $self}             = {};
-        $_tid{refaddr $self}                  = qq[\0] x 5;
-        $_use_dht{refaddr $self}              = 1;
-        $_dht{refaddr $self} = Net::BitTorrent::DHT->new({Client => $self});
-        $_peerid{refaddr $self}      = Net::BitTorrent::Version::gen_peerid();
-        $_connections{refaddr $self} = {};
+
+        # Try opening a matching set of ports
+        for my $port (@ports) {
+            last
+                if $self->_socket_open_tcp($host, $port)
+                    && $self->_socket_open_udp($host, $port);
+        }
+
+        # Clear everything just in case
         $self->_reset_bandwidth;
-        for my $port (@ports) { last if $self->_socket_open($host, $port) }
         weaken($REGISTRY{refaddr $self} = $self);
+        $$self = $_peerid{refaddr $self};
         return $self;
     }
 
@@ -78,8 +92,12 @@ package Net::BitTorrent;
     sub _connections { return $_connections{refaddr +shift} }
     sub _max_ul_rate { return $_max_ul_rate{refaddr +shift} }
     sub _max_dl_rate { return $_max_dl_rate{refaddr +shift} }
-    sub _dht { my ($s) = @_; return $_udp{refaddr $s} && $_dht{refaddr $s} }
-    sub _use_dht { return $_use_dht{refaddr +shift}; }
+    sub _dht         { return $_dht{refaddr +shift} }
+
+    sub _use_dht {
+        my ($s) = @_;
+        return $_udp{refaddr $s} && $_use_dht{refaddr $s};
+    }
 
     sub _tcp_port {
         my ($self) = @_;
@@ -233,29 +251,30 @@ package Net::BitTorrent;
         return delete $_connections{refaddr $self}{fileno $socket};
     }
 
-    sub _socket_open {
+    sub _socket_open_tcp {
         my ($self, $host, $port) = @_;
         if (   not $self
             || not blessed $self
             || not $self->isa(q[Net::BitTorrent]))
         {   carp
-                q[Net::BitTorrent->_socket_open(HOST, PORT) requires a blessed object];
+                q[Net::BitTorrent->_socket_open_tcp(HOST, PORT) requires a blessed object];
             return;
         }
         if ((!$_tcp{refaddr $self}) && (!$host)) {
-            carp q[Net::BitTorrent::__socket_open( ) ]
+            carp q[Net::BitTorrent::_socket_open_tcp( ) ]
                 . q[requires a hostname];
             return;
         }
-        if (   (!$_tcp{refaddr $self}) && (!defined $port)
-            || (defined $port and $port !~ m[^\d+$]))
-        {   carp q[Net::BitTorrent::__socket_open( ) ]
+        if (defined $port and $port !~ m[^\d+$]) {
+            carp q[Net::BitTorrent::_socket_open_tcp( ) ]
                 . q[requires an integer port number];
             return;
         }
         my $_packed_host = undef;
         $host ||= q[0.0.0.0];
         $port ||= 0;
+        $port =~ m[^(\d+)$];
+        $port = $1;
         if (    $host
             and $host
             !~ m[^(?:(?:(?:25[0-5]|2[0-4][0-9]|[0-1]?[0-9]{1,2})[.]?){4})$])
@@ -265,45 +284,100 @@ package Net::BitTorrent;
             $_packed_host = $addrs[0];
         }
         else { $_packed_host = inet_aton($host) }
-        if (!$_tcp{refaddr $self}) {
-            socket(my ($_tcp), PF_INET, SOCK_STREAM, getprotobyname(q[tcp]))
-                or return;
-            bind($_tcp, pack_sockaddr_in($port, $_packed_host))
-                or return;
-            listen($_tcp, 1) or return;
-            $_connections{refaddr $self}{fileno $_tcp} = {Object => $self,
-                                                          Mode   => q[ro]
-                }
-                or return;
-            $_tcp{refaddr $self} = $_tcp;
+        socket(my ($_tcp), PF_INET, SOCK_STREAM, getprotobyname(q[tcp]))
+            or return;
+
+       # - What is the difference between SO_REUSEADDR and SO_REUSEPORT?
+       #    [http://www.unixguide.net/network/socketfaq/4.11.shtml]
+       # - setsockopt - what are the options for ActivePerl under Windows NT?
+       #    [http://perlmonks.org/?node_id=63280]
+       #      setsockopt($_tcp, SOL_SOCKET, SO_REUSEADDR, pack(q[l], 1))
+       #         or return;
+       # SO_REUSEPORT is undefined on Win32... Boo...
+       #if ($reuse_port and defined SO_REUSEPORT) {       # XXX - undocumented
+       #   setsockopt($_udp, SOL_SOCKET, SO_REUSEPORT, pack(q[l], 1))
+       #       or return;
+       #}
+        bind($_tcp, pack_sockaddr_in($port, $_packed_host))
+            or return;
+        listen($_tcp, 1) or return;
+        $_connections{refaddr $self}{fileno $_tcp} = {Object => $self,
+                                                      Mode   => q[ro],
+            }
+            or return;
+        if (   defined $_tcp{refaddr $self}
+            && fileno $_tcp{refaddr $self}
+            && defined $_connections{refaddr $self}
+            {fileno $_tcp{refaddr $self}})
+        {   delete $_connections{refaddr $self}{fileno $_tcp{refaddr $self}};
+            close $_tcp{refaddr $self};
         }
-        if ($_tcp{refaddr $self}
-            and !$_udp{refaddr $self})
-        {   ($port, undef)
-                = unpack_sockaddr_in(getsockname($_tcp{refaddr $self}));
-            socket(my ($_udp), PF_INET, SOCK_DGRAM, getprotobyname(q[udp]))
-                or return;
-            bind($_udp, pack_sockaddr_in($port, $_packed_host))
-                or return;
-            $_connections{refaddr $self}{fileno $_udp} = {Object => $self,
-                                                          Mode   => q[ro]
-                }
-                or return;
-            $_udp{refaddr $self} = $_udp;
+        return $_tcp{refaddr $self} = $_tcp;
+    }
+
+    sub _socket_open_udp {
+        my ($self, $host, $port) = @_;
+        if (   not $self
+            || not blessed $self
+            || not $self->isa(q[Net::BitTorrent]))
+        {   carp
+                q[Net::BitTorrent->_socket_open_udp(HOST, PORT) requires a blessed object];
+            return;
         }
-        if (    defined $_tcp{refaddr $self}
-            and defined $_udp{refaddr $self})
-        {   my ($tcp_port, $tcp_packed_ip)
-                = unpack_sockaddr_in(getsockname($_tcp{refaddr $self}));
-            my $tcp_address
-                = sprintf(q[%s:%d], inet_ntoa($tcp_packed_ip), $tcp_port);
-            my ($udp_port, $udp_packed_ip)
-                = unpack_sockaddr_in(getsockname($_udp{refaddr $self}));
-            my $udp_address
-                = sprintf(q[%s:%d], inet_ntoa($udp_packed_ip), $udp_port);
-            $$self = $tcp_address . q[ | ] . $udp_address;
+        if ((!$_tcp{refaddr $self}) && (!$host)) {
+            carp q[Net::BitTorrent::_socket_open_udp( ) ]
+                . q[requires a hostname];
+            return;
         }
-        return ($_tcp{refaddr $self} && $_udp{refaddr $self});
+        if (defined $port and $port !~ m[^\d+$]) {
+            carp q[Net::BitTorrent::_socket_open_udp( ) ]
+                . q[requires an integer port number];
+            return;
+        }
+        my $_packed_host = undef;
+        $host ||= q[0.0.0.0];
+
+        #$port = $port ? $port : $_udp{refaddr $self} ? $self->_udp_port : 0;
+        $port ||= 0;
+        $port =~ m[^(\d+)$];
+        $port = $1;
+        if (    $host
+            and $host
+            !~ m[^(?:(?:(?:25[0-5]|2[0-4][0-9]|[0-1]?[0-9]{1,2})[.]?){4})$])
+        {   my ($name, $aliases, $addrtype, $length, @addrs)
+                = gethostbyname($host)
+                or return;
+            $_packed_host = $addrs[0];
+        }
+        else { $_packed_host = inet_aton($host) }
+        socket(my ($_udp), PF_INET, SOCK_DGRAM, getprotobyname(q[udp]))
+            or return;
+
+       # - What is the difference between SO_REUSEADDR and SO_REUSEPORT?
+       #    [http://www.unixguide.net/network/socketfaq/4.11.shtml]
+       # - setsockopt - what are the options for ActivePerl under Windows NT?
+       #    [http://perlmonks.org/?node_id=63280]
+       #     setsockopt($_udp, SOL_SOCKET, SO_REUSEADDR, pack(q[l], 1))
+       #        or return;
+       # SO_REUSEPORT is undefined on Win32... Boo...
+       #if ($reuse_port and defined SO_REUSEPORT) {       # XXX - undocumented
+       #   setsockopt($_udp, SOL_SOCKET, SO_REUSEPORT, pack(q[l], 1))
+       #       or return;
+       #}
+        bind($_udp, pack_sockaddr_in($port, $_packed_host))
+            or return;
+        $_connections{refaddr $self}{fileno $_udp} = {Object => $self,
+                                                      Mode   => q[ro],
+            }
+            or return;
+        if (   $_udp{refaddr $self}
+            && fileno $_udp{refaddr $self}
+            && defined $_connections{refaddr $self}
+            {fileno $_udp{refaddr $self}})
+        {   delete $_connections{refaddr $self}{fileno $_udp{refaddr $self}};
+            close $_udp{refaddr $self};
+        }
+        return $_udp{refaddr $self} = $_udp;
     }
 
     sub _process_connections {
@@ -592,21 +666,25 @@ package Net::BitTorrent;
 
     sub as_string {
         my ($self, $advanced) = @_;
-        my $dump = !$advanced ? $_peerid{refaddr $self} : sprintf <<'END',
+        my $dump
+            = !$advanced ? $_peerid{refaddr $self} : sprintf <<'END',
 Net::BitTorrent
 
 Peer ID: %s
-DHT is %sabled
+DHT is %sabled (Node ID: %s)
 TCP Address: %s:%d
 UDP Address: %s:%d
 ----------
 Torrents in queue: %d
 %s
+----------
 END
             $_peerid{refaddr $self},
-            ($_use_dht{refaddr $self} ? q[En] : q[Dis]),
+            $_use_dht{refaddr $self} ? q[En] : q[Dis],
+            unpack(q[H*], $_dht{refaddr $self}->node_id),
             $self->_tcp_host, $self->_tcp_port, $self->_udp_host,
-            $self->_udp_port, (scalar keys %{$_torrents{refaddr $self}}),
+            $self->_udp_port,
+            (scalar keys %{$_torrents{refaddr $self}}),
             join(qq[\r\n], keys %{$_torrents{refaddr $self}});
         return defined wantarray ? $dump : print STDERR qq[$dump\n];
     }
@@ -627,6 +705,11 @@ END
     }
     DESTROY {
         my ($self) = @_;
+        close($_tcp{refaddr $self}) if $_tcp{refaddr $self};
+        close($_udp{refaddr $self}) if $_udp{refaddr $self};
+        foreach my $conn (values %{$_connections{refaddr $self}}) {
+            close($conn->{q[Object]}->_socket) if $conn->{q[Object]};
+        }
         for (@CONTENTS) { delete $_->{refaddr $self}; }
         return delete $REGISTRY{refaddr $self};
     }
@@ -707,23 +790,6 @@ C<undef> or an empty list.
 
 =over 4
 
-=item C<peerid ( )>
-
-Returns the L<Peer ID|Net::BitTorrent::Version/"gen_peerid ( )">
-generated to identify this L<Net::BitTorrent|Net::BitTorrent> object
-internally, with remote L<peers|Net::BitTorrent::Peer>, and
-L<trackers|Net::BitTorrent::Torrent::Tracker>.
-
-See also: wiki.theory.org (http://tinyurl.com/4a9cuv),
-L<Peer ID Specification|Net::BitTorrent::Version/"Peer ID Specification">
-
-=item C<torrents ( )>
-
-Returns the list of queued L<torrents|Net::BitTorrent::Torrent>.
-
-See also: L<add_torrent ( )|/"add_torrent ( { ... } )">,
-L<remove_torrent ( )|/"remove_torrent ( TORRENT )">
-
 =item C<add_torrent ( { ... } )>
 
 Loads a .torrent file and adds the
@@ -743,23 +809,6 @@ See also: L<torrents ( )|/"torrents ( )">,
 L<remove_torrent ( )|/"remove_torrent ( TORRENT )">,
 L<Net::BitTorrent::Torrent|Net::BitTorrent::Torrent>
 
-=item C<remove_torrent ( TORRENT )>
-
-Removes a L<Net::BitTorrent::Torrent|Net::BitTorrent::Torrent> object
-from the client's queue.
-
-=begin future
-
-Before the torrent torrent is closed, we announce to the tracker that we
-have 'stopped' downloading and a callback to store the current state is
-called.
-
-=end future
-
-See also: L<torrents ( )|/"torrents ( )">,
-L<add_torrent ( )|/"add_torrent ( { ... } )">,
-L<Net::BitTorrent::Torrent|Net::BitTorrent::Torrent>
-
 =item C<do_one_loop ( [TIMEOUT] )>
 
 Processes the internal schedule and handles activity of the various
@@ -782,6 +831,40 @@ from a file, use C<$client-E<gt>on_event( 'file_read', \&on_read )>.
 See the L<Events|/Events> section for a list of events sorted by their
 related classes.
 
+=item C<peerid ( )>
+
+Returns the L<Peer ID|Net::BitTorrent::Version/"gen_peerid ( )">
+generated to identify this L<Net::BitTorrent|Net::BitTorrent> object
+internally, with remote L<peers|Net::BitTorrent::Peer>, and
+L<trackers|Net::BitTorrent::Torrent::Tracker>.
+
+See also: wiki.theory.org (http://tinyurl.com/4a9cuv),
+L<Peer ID Specification|Net::BitTorrent::Version/"Peer ID Specification">
+
+=item C<remove_torrent ( TORRENT )>
+
+Removes a L<Net::BitTorrent::Torrent|Net::BitTorrent::Torrent> object
+from the client's queue.
+
+=begin future
+
+Before the torrent torrent is closed, we announce to the tracker that we
+have 'stopped' downloading and a callback to store the current state is
+called.
+
+=end future
+
+See also: L<torrents ( )|/"torrents ( )">,
+L<add_torrent ( )|/"add_torrent ( { ... } )">,
+L<Net::BitTorrent::Torrent|Net::BitTorrent::Torrent>
+
+=item C<torrents ( )>
+
+Returns the list of queued L<torrents|Net::BitTorrent::Torrent>.
+
+See also: L<add_torrent ( )|/"add_torrent ( { ... } )">,
+L<remove_torrent ( )|/"remove_torrent ( TORRENT )">
+
 =back
 
 =head1 Events
@@ -811,7 +894,7 @@ the following key:
 
 =item C<Address>
 
-IPv4:port address of the potential peer.
+IPv4:port (or, on rare occasions, hostname:port) address of the potential peer.
 
 =back
 
@@ -1336,6 +1419,6 @@ clarification, see http://creativecommons.org/licenses/by-sa/3.0/us/.
 Neither this module nor the L<Author|/Author> is affiliated with
 BitTorrent, Inc.
 
-=for svn $Id: BitTorrent.pm 42 2008-12-05 04:54:43Z sanko@cpan.org $
+=for svn $Id: BitTorrent.pm 45 2008-12-26 22:17:16Z sanko@cpan.org $
 
 =cut
