@@ -10,8 +10,8 @@ package Net::BitTorrent::Torrent::Tracker;
     use Net::BitTorrent::Torrent::Tracker::HTTP;
     use Net::BitTorrent::Torrent::Tracker::UDP;
     use version qw[qv];
-    our $SVN = q[$Id: Tracker.pm 40 2008-12-02 04:25:26Z sanko@cpan.org $];
-    our $UNSTABLE_RELEASE = 0; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new((qw$Rev: 40 $)[1])->numify / 1000), $UNSTABLE_RELEASE);
+    our $SVN = q[$Id: Tracker.pm 42 2008-12-05 04:54:43Z sanko@cpan.org $];
+    our $UNSTABLE_RELEASE = 0; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new((qw$Rev: 42 $)[1])->numify / 1000), $UNSTABLE_RELEASE);
     my (@CONTENTS) = \my (%torrent, %_urls, %complete, %incomplete);
     my %REGISTRY;
 
@@ -95,7 +95,7 @@ package Net::BitTorrent::Torrent::Tracker;
         return $_urls{refaddr $self}->[0]->_announce($event ? $event : ());
     }
 
-    sub _as_string {
+    sub as_string {
         my ($self, $advanced) = @_;
         my $dump = !$advanced ? $$self : sprintf <<'END',
 Net::BitTorrent::Torrent::Tracker
@@ -162,6 +162,12 @@ swarm.
 Returns the number of incomplete peers the tracker says are present in
 the swarm.
 
+=item C<as_string ( [ VERBOSE ] )>
+
+Returns a 'ready to print' dump of the  object's data structure.  If
+called in void context, the structure is printed to C<STDERR>.
+C<VERBOSE> is a boolean value.
+
 =back
 
 =head1 Author
@@ -188,6 +194,6 @@ clarification, see http://creativecommons.org/licenses/by-sa/3.0/us/.
 Neither this module nor the L<Author|/Author> is affiliated with
 BitTorrent, Inc.
 
-=for svn $Id: Tracker.pm 40 2008-12-02 04:25:26Z sanko@cpan.org $
+=for svn $Id: Tracker.pm 42 2008-12-05 04:54:43Z sanko@cpan.org $
 
 =cut
